#ifndef PLAYLIST_H
#define PLAYLIST_H

#include <noatun/playlist.h>
#include <noatun/plugin.h>
#include "flexprefs.h"

class SafeListViewItem;
class View;
class List;
class QListViewItem;

class FlexPlaylist : public Playlist, public Plugin, PlaylistNotifier
{
	Q_OBJECT
	friend class SafeListViewItem;
	friend class List;

	public:
		FlexPlaylist();
		~FlexPlaylist();

		/**
		* go to the front
		**/
		virtual void reset();

		virtual void clear();
		virtual void addFile(const KURL&, bool play=false);
		/**
		* Cycle everthing through forward
		**/
		virtual PlaylistItem next();
		PlaylistItem next(bool play);
		/**
		* return the one that might/should be playing now
		**/
		virtual PlaylistItem current();
		/**
		* Cycle through backwards
		**/
		virtual PlaylistItem previous();

		virtual PlaylistItem getFirst() const;
		virtual PlaylistItem getAfter(const PlaylistItem &item) const;

		virtual bool listVisible() const;
		virtual void init();

		/**
		* Called when a playlistitem changed length
		* Needed to compute length of whole playlist
		**/
		void lengthChanged(QString &, QString &);

		virtual Playlist *playlist() { return this; }
		FlexPrefs *prefs() const { return mPrefs; }

		static FlexPlaylist *FPL() { return Self; }
		const unsigned long fullLength() { return mFullLength; }
		inline bool exiting() const { return mExiting; }

		// from playlistnotifier
//		virtual void added(PlaylistItem &);
		virtual void removed(PlaylistItem &);
//		virtual void modified(PlaylistItem &);

	public slots:
		virtual void showList();
		virtual void hideList();
		virtual void remove(const PlaylistItem&);
		virtual void sort();


	public slots:
		//void setNext(const PlaylistItem &);
		void setCurrent(const PlaylistItem &, bool emitC);
		void setCurrent(const PlaylistItem &);
		//void setPrevious(const PlaylistItem &);

		void listItemSelected(QListViewItem*);

		void randomize();

	signals:
		void play(PlaylistItem*);

	private:
		unsigned long stringToLen(QString &);

	private:
		//PlaylistItem nextItem, currentItem, previousItem;
		PlaylistItem currentItem, randomPrevious;
		bool mExiting; // HACK HACK HACK HACK!!!
		View *view;
		static FlexPlaylist *Self;
		FlexPrefs *mPrefs;
		unsigned long mFullLength;
};
#endif
